/*************************************************************************

DiscEx

PC disk excerciser

Use this program to burn in a disk drive or test for intermittent errors
or to force a "dying" disk to fail.

Usage:  DiscEx  [ <drive> [ <no passes> ] ]

        <drive> is the drive on which the test is to be performed.
        
        <no passes> in the number of times you want DiscEx to execute
        the random seek test.  Each pass executes 500 random positions
        and reads.
        
        The test is performed on the specified drive.  If no drive is
        specified, then the test is on the current (default) drive.
        
License and copyright:

        The program (DiscEx.exe and DiscEx.c) is copyright (c) 1988,1989 by 
        Steven D. Stern.  
        
        This utility is provided FREE but IS NOT placed in the public
        domain.  You may freely copy and distribute DISCEX provided 
        it is distributed as a package containing UNMODIFIED copies of
        
           DISCEX.EXE
           DISCEX.C
           DISCEX.DOC
           
        The author assumes no liability (direct, consequential, or otherwise)
        and provides absolutely no warranty or guarantees regarding this utility.
        Use it at your own risk.
        
        The author may contacted via Compuserve:  70327,135

        Compilation Notes:  A large stack is required.  Use approx.
                            18K.
*************************************************************************/
        
          
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <memory.h>
#include <sys\timeb.h>
#include <signal.h>

#define BUFSIZE 16384

long random();

char filename[128];
FILE *testfile;

main(int argc, char *argv[] )
   {
   long bytes,                  /* max file size in bytes */
        blocks,                 /* max no. of BUFSIZE blocks */
        position;
   long dpi;
   int ret;
   char driveltr;              // user specified drive name
   int  drivenum;              // user specified drive no
   int pass=1;                 /* no of times to repeat random reads */
   struct timeb TIME;   
   struct diskfree_t drive;
   char filebuf[BUFSIZE];
   int break_handler();         // handles ^C
   
   if (argc > 1)
      {
      driveltr = *argv[1];
      drivenum = (toupper(driveltr)-'A')+1;
      }
     else
      {
      _dos_getdrive(&drivenum);
      driveltr = drivenum + 'A' - 1;
      drivenum = 0;
      }
      
   if (argc > 2)
      {
      pass = atoi(argv[2]);
      }

   printf("DiscEx:  The PC disk excerciser (1.03)\n");
   printf("Copyright (c) 1988, 1989, Steven D. Stern\n\n");
   
   _dos_getdiskfree(drivenum,&drive);   /* how much free space on disk? */
   
   bytes =                       /* convert it to byte-size      */
      (long)drive.bytes_per_sector * 
      (long)drive.sectors_per_cluster * 
      (long)drive.avail_clusters;
   
   *filebuf = driveltr;
   strcpy(&filebuf[1],":\\");   
   strcat(filebuf,"DXXXXXXX");
   
   mktemp(filebuf);
   
   testfile = fopen(filebuf,"a+b");
   
   if (testfile == NULL)
      {
      printf("Unable to open scratch file: %s\n",filebuf);
      exit(2);
      }
   
   signal(SIGINT,break_handler);    // assign the ^C handler
   printf("Press Ctrl-C or Ctrl-Break to terminate...\n\n");
   
   printf("Creating scratch file: %s\n",filebuf);
   strcpy(filename,filebuf);
   
   setvbuf(testfile,NULL,_IOFBF,BUFSIZE);
   
   blocks = bytes / BUFSIZE;
   
   memset(filebuf,0xF5,BUFSIZE);        /* load buf with some data */
   
   printf("Loading scratch file for %ld bytes\n",blocks*BUFSIZE);
   
   for (dpi = 0; dpi < blocks; dpi++)
      {
      ret = fwrite(filebuf,BUFSIZE,1,testfile);
      if (ret != 1)
         {
         printf("Error writing scratch file\n");
         fclose(testfile);
         unlink(filename);
         exit(1);
         }
      }
   fflush(testfile);
   printf("Load successful\n");
   
   printf("Beginning sequential read test\n");
   
   fseek(testfile,0L,SEEK_SET);         /* rewind the file */
   for (dpi = 0; dpi < blocks; dpi++)
      {
      ret = fread(filebuf,BUFSIZE,1,testfile);
      }
   printf("Sequential read test complete\n");
   
   setvbuf(testfile,NULL,_IONBF,0);     /* turn off buffering */
   printf("Beginning random seek test...\n");
   
   while (pass)
      {
      ftime(&TIME);
      srand(TIME.millitm);
      printf("%d pass(es) remaining",pass);
      fflush(stdout);
      for (dpi = 0; dpi < 500L; dpi++)
         {
         position = random(bytes);
         fseek(testfile,position,SEEK_SET);
         fread(filebuf,1,1,testfile);
         }
      printf("... complete\n");
      pass--;
      }
   printf("Random seek test complete\n");
   printf("Erasing scratch file\n");
   
   fclose(testfile);
   unlink(filename);
   }

long random( long top )
   {
   double scale;
   int i;
   double d;
   
   i = rand();
   scale = (double)top / (double)32768;
   d= scale * (double) i;
   if ( (long)d > top)
      {
      printf("Random problem\n");
      return (0L);
      }
   return ( (long)d );
   }
   
int break_handler()
   {
   signal(SIGINT,SIG_IGN);    // disable break while processing a break
   fclose(testfile);          // close the scratch file
   unlink(filename);          // and purge it
   printf("\n\nControl-C interrupt.\n");
   printf("Execution terminated; scratch file deleted.");
   exit(1);
   }

